/*
 * SetFile.cpp
 *
 * David Evans (dfevans@bbcr.uwaterloo.ca), July 5th, 1996
 *
 * Browser add-on to run setfile on the indicated refs.
 * Shameless modification of the "MakeArchive" add-on supplied
 * by Be.
 */

#include <Directory.h>
#include <Window.h>
#include <Rect.h>
#include <View.h>
#include <File.h>
#include <Alert.h>
#include <AppDefs.h>
#include <Message.h>
#include <Errors.h>

#include <Debug.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>


extern char** environ;

class TStatusView : public BView
{
public:
					TStatusView(BRect);
virtual void		Draw(BRect);
};

class TStatusWindow : public BWindow
{
public:
					TStatusWindow();
};

#pragma export on
void process_refs(record_ref dir_ref, BMessage* msg, void*);
#pragma export reset

void process_refs(record_ref dir_ref, BMessage* msg, void*)
{

	BAlert*			alert;
	char**			eargv;
	TStatusWindow*	wind;
	ulong			type;
	record_ref		ref;
	long			result;
	thread_info		info;
	long			count;
	long			i;
	BFile			file;
	BDirectory		dir;
	char			path[1000];
	char			name[B_FILE_NAME_LENGTH];
	thread_id		tid;

	// this add-on requires a selection
	msg->GetInfo("refs", &type, &count);
	if (count == 0)
	{
		alert = new BAlert("", "You must have a selection to setfile.", "Cancel");
		alert->Go();
		return;
	}

	// Create argv style buffer to pass to setfile
	// Need a string for the command name and the parameter and the zero.
	eargv = (char**)malloc(3 * (sizeof(char*)));

	for (i = 0; i < count; i++)
	{
		ref = msg->FindRef("refs", i);
		if (does_ref_conform(ref, "File"))
			file.SetRef(ref);
		else if(does_ref_conform(ref, "Folder"))
		{
			dir.SetRef(ref);
			dir.GetName(name);
			sprintf(path, "Sorry, can't setfile %s because it is a folder.", name);
			alert = new BAlert("", path, "Cancel");
			alert->Go();
			continue;
		}
		else
		{
			alert = new BAlert("", "Sorry, can't setfile database-only items.", "Cancel");
			alert->Go();
			continue;;
		}
			
		file.GetPath(path, 1000);
		eargv[1] = (char*)malloc(strlen(path) + 1);
		strcpy(eargv[1], path);
		eargv[0] = "/boot/bin/setfile";
		eargv[2] = NULL;
		
		// Launch setfile
		tid = load_posix(eargv[0], eargv, environ);
		if (tid > 0)
		{
			resume_thread(tid);
	
			// if the thread is still around then bring up status window
			if (get_thread_info(tid, &info) == B_NO_ERROR) {
				wind = new TStatusWindow();
				wait_for_thread(tid, &result);
				wind->Close();
			}
		}
		else
		{
			alert = new BAlert("", "Sorry, could not find the setfile tool.", "Cancel");
			alert->Go();
		}
		
		// Free the path statement
		free(eargv[1]);
	}
}

// ---------------------------------------------------------------

TStatusView::TStatusView(BRect r)
		    :BView(r, "", B_FOLLOW_NONE, B_WILL_DRAW | B_PULSE_NEEDED)
{
}

// ---------------------------------------------------------------

void TStatusView::Draw(BRect)
{
	SetFontName("Erich");
	SetFontSize(9);
	MovePenTo(10, 20);
	DrawString("Running setfile...");
}

// ---------------------------------------------------------------


TStatusWindow::TStatusWindow()
			  :BWindow(BRect(100, 120, 300, 160), "Add-on Status", B_TITLED_WINDOW,
					   B_NOT_CLOSABLE | B_NOT_RESIZABLE)
{
	BRect	status_wind_rect(100, 120, 300, 160);

	status_wind_rect.OffsetTo(B_ORIGIN);
	AddChild(new TStatusView(status_wind_rect));
	Show();
}
